@@ -165,6 +165,7 @@ init_validator ()
                      "|(get_reports)"
                      "|(get_report_format)"
                      "|(get_report_formats)"
+                     "|(get_resource_names)"
                      "|(get_result)"
                      "|(get_results)"
                      "|(get_role)"
@@ -429,6 +430,7 @@ init_validator ()
                      "^(summary|results|hosts|ports"
                      "|closed_cves|os|apps|errors"
                      "|topology|ssl_certs|cves)$");
+  gvm_validator_add (validator, "resource_type", "(?s)^.*$");
   gvm_validator_add (validator, "result_id", "^[a-z0-9\\-]+$");
   gvm_validator_add (validator, "role", "^[[:alnum:] ]+$");
   gvm_validator_add (validator, "permission", "^([_a-z]+|Super)$");
